(* Tao Zha's Mathematica Library *)

FormatMatlabForMatrixOutput[mat_?MatrixQ, ArrayName_String] :=
 Module[{m, n},
  {m, n} = Dimensions[mat];
  Flatten[Table[
    ArrayName <> ToString[StringForm[
       "(`1`,`2`) = `3`;",
       i,
       j,
       mat[[i, j]] // N // CForm
       ]],
    {i, m},
    {j, n}]
   ]
  ]

FormatMatlabForVectorOutput[vec_?VectorQ, ArrayName_String] :=
 Table[
  ArrayName <> ToString[StringForm[
     "(`1`) = `2`;",
     i,
     vec[[i]] // N // CForm
     ]],
  {i, Length[vec]}
  ]

FormatMatlabForOutput[vec_?VectorQ, ArrayName_String] :=
 Table[
  ArrayName <> ToString[StringForm[
     "(`1`) = `2`;",
     i,
     vec[[i]] // N // CForm
     ]],
  {i, Length[vec]}
  ]

FormatCForOutput[mat_?MatrixQ, ArrayName_String] :=
 Module[{m, n},
  {m, n} = Dimensions[mat];
  Flatten[Table[
    ArrayName <> ToString[StringForm[
       "[`1`,`2`] = `3`;",
       i - 1,
       j - 1,
       mat[[i, j]] // N // CForm
       ]],
    {i, m},
    {j, n}]
   ]
  ]
FormatCForOutput[vec_?VectorQ, ArrayName_String] :=
 Table[
  ArrayName <> ToString[StringForm[
     "[`1`] = `2`;",
     i - 1,
     vec[[i]] // N // CForm
     ]],
  {i, Length[vec]}
  ]

(* ----------------------------------------------------- *)
FormatForGreekLettersOutput[vec_?VectorQ] :=
 Table[
  ToString[StringForm[
    "`1` = ;",
    vec[[i]] // N // CForm
    ]],
  {i, Length[vec]}]

FormatCMatlabForEqualityOutput[LeftHandList_, RightHandList_] :=
  Table[ToString[StringForm[
     "`1` = `2`;",
     LeftHandList[[i]],
     RightHandList[[i]] // N // CForm
     ]],
   {i, Length[LeftHandList]}
   ];


(* --------------------- Replace strings -------------------------------- *)
(* Outputs are all strings. *)
TranslatingToNoSubscript[coef_, rules_] :=
 coef /. rules /.
  Subscript[x_, y_] :> ToExpression[ToString[x] <> ToString[y]]

MatlabStringReplace[str_] :=
 StringReplace[
  str, {Shortest["Power(" ~~ x__ ~~ "," ~~ y__ ~~ ")"] :>
    "(" ~~ x ~~ ")" ~~ "^" ~~ "(" ~~ y ~~ ")", 
    "Log(" -> "log("}]

CStringReplace[str_] :=
 StringReplace[str, {"Power(" -> "pow(", "Sqrt" -> "sqrt","Log(" -> "log("}]

(* ------------------------ Append string to the file ---------------------------------- *)
ExportMatlabStringAppend::usage =
  "ExportMatlabStringAppend[filename, data] imports Lines from \
filename and joins the data (a list of strings) and exports to \
filename. ExportAppend creates the file if filename does not exist.";

ExportMatlabStringAppend[filename_String, data_] :=
 Module[{divider0 = {"\n%-------------------\n"}},
  Export[filename,
    Join[
     Quiet[Check[Import[filename, "Lines"], {}]],
     divider0,
     data],
    "Text"];
  ]
 
  
ExportCStringAppend::usage =
  "ExportCStringAppend[filename, data] imports Lines from \
filename and joins the data (a list of strings) and exports to \
filename. ExportAppend creates the file if filename does not exist.";

ExportCStringAppend[filename_String, data_] :=
 Module[{divider0 = {"\n//-------------------\n"}},
  Export[filename,
    Join[
     Quiet[Check[Import[filename, "Lines"], {}]],
     divider0,
     data],
    "Text"];
  ]



(* ------------------- Arranging the measurement equations into matrix measurement form.  See the project example with Marvin Goodfriend. ---------------- *) 
tzMeasurementForm[equations:{__Equal}, xstatet_List, xut_List] :=
 Module[{a, H, Psiu},    
  {resid, H} = Normal[CoefficientArrays[equations[[All, 2]], xstatet]];
  {a, Psiu} = Normal[CoefficientArrays[resid, xut]];
    (* All means all the rows; [All, 2] means right-hand equations for all the rows. *)
  {a, H, Psiu}
 ] 

tzMeasurementFormNoMeasureErrors[equations:{__Equal}, xstatet_List] :=
Module[{a, H}, 
 {a, H} = Normal[
   CoefficientArrays[equations[[All, 2]], xstatet]]; 
   (* All means all the rows; [All, 2] means right-hand equations for all the rows. *)
 {a, H}
]

            
tzMeasurementForm4HOnly[equations:{__Equal}, xstatet_List] :=
Module[{H}, 
 H = Normal[
   CoefficientArrays[equations[[All, 2]], xstatet][[2]]]; 
   (* All means all the rows; [All, 2] means right-hand equations for all the rows. *)
 {H}
] 


(* ------------------- Arranging the state equations into matrix state form ---------------- *)
tzStateForm[equations:{__Equal}, y0_List, \[CurlyEpsilon]0_List, t_: t] := 
 Module[{y1, \[CurlyEpsilon]1, \[CurlyEpsilon], resid, g0, g1, psi, c},
    y1 = y0 /. t -> t - 1;
    \[CurlyEpsilon]1 = 
     Union[Cases[equations, 
       Alternatives @@ (\[CurlyEpsilon]0 /. t -> t - 1), Infinity]];
    \[CurlyEpsilon] = Join[\[CurlyEpsilon]0, \[CurlyEpsilon]1];
    {resid, g0} = Normal[CoefficientArrays[equations, y0]];
    resid = -resid;
    {g1, psi} = 
     Normal[CoefficientArrays[resid, #][[2]] & /@ {y1, \[CurlyEpsilon]}];
    c = resid - (g1.y1 + psi.\[CurlyEpsilon]) // Simplify;
    {{g0, g1, c, psi}, {y0, y1, \[CurlyEpsilon]}} 
 ]
  
  
  
(* ------------------- Exporting Greek symbols to ascii symbols used by C or Matlab  ---------------- *)
(* ------------------- Calling example: BaseParsOuput = ExportSymbols[BasePars,"tz"] ---------------- *)
ExportSymbols[parmlist_List, initial_: ""] := 
 ToExpression[initial <> # & /@ StringReplace[(ToString[FullForm[#]] & /@ parmlist),
    {"Subscript" -> "", "\\[" -> "", "]" -> "","[" -> "", "," -> "", " " -> ""}]
 ] 

(* ------------------- Rules for exporting Greek symbols to ascii symbols used by C or Matlab ---------------- *)
(* ------------------- Calling example: BaseParsOuput = ExportSymbolsRules[BasePars,"tz"]     ---------------- *)
ExportSymbolsRules[parmlist_List, initial_: ""] := 
 Thread[parmlist -> ExportSymbols[parmlist, initial]]
  
ConvertBase1ToBase0[x_String, y_String] :=
 Module[{x1, y1},
  x1 = ToString[ToExpression[x] - 1];
  y1 = ToString[ToExpression[y] - 1];
  "(" <> x1 <> "," <> y1 <> ")"
 ] 
 
ConvertBase1ToBase0Vec[x_String] :=
 Module[{x1},
  x1 = ToString[ToExpression[x] - 1];
  "(" <> x1 <> ")"
 ] 
   
RewriteGensysMatrices0[fileIn_String, fileOut_String] :=
 Module[{in1, in2, in3},
  in1 = Import[fileIn, "Text"];
  in2 = StringReplace[
    in1, {"A" | "B" | "CapitalGammau" | "CapitalGammaeta" ~~ "(" ~~ 
       DigitCharacter .. ~~ "," ~~ DigitCharacter .. ~~ ") = 0.;\n" ->
       ""}];
  in3 = StringReplace[in2, 
    "(" ~~ x : DigitCharacter .. ~~ "," ~~ y : DigitCharacter .. ~~ 
      ")" :> ConvertBase1ToBase0[x, y]];
  Export[fileOut, in3, "Text"]
 ]  
 
RewriteGensysMatrices[fileIn_String, fileOut_String] :=
 Module[{in1, in2, in3},
  in1 = Import[fileIn, "Text"];
  in2 = StringReplace[
    in1, {"A" | "B" | "CapitalGammau" | "CapitalGammaeta" ~~ "(" ~~ 
       DigitCharacter .. ~~ "," ~~ DigitCharacter .. ~~ ") = 0.;\n" ->
       "", 
       "cgensys" ~~ "(" ~~ DigitCharacter .. ~~ ") = 0.;\n" -> 
          ""}];
  in3 = StringReplace[in2, 
    {"(" ~~ x : DigitCharacter .. ~~ "," ~~ y : DigitCharacter .. ~~ 
      ")" :> ConvertBase1ToBase0[x, y], 
      "(" ~~ x : DigitCharacter .. ~~ ")" :> 
         ConvertBase1ToBase0Vec[x]}];
  Export[fileOut, in3, "Text"]
 ]  
   
RewriteMeasurementMatrices[fileIn_String, fileOut_String] := 
 Module[{in1, in2, in3}, in1 = Import[fileIn, "Text"];
  in2 = StringReplace[
    in1, {"ameasure" ~~ "(" ~~ DigitCharacter .. ~~ ") = 0.;\n" -> 
      "", "Hmeasure" ~~ "(" ~~ DigitCharacter .. ~~ "," ~~ 
       DigitCharacter .. ~~ ") = 0.;\n" -> ""}];
  in3 = StringReplace[
    in2, {"(" ~~ x : DigitCharacter .. ~~ ")" :> 
      ConvertBase1ToBase0Vec[x], 
     "(" ~~ x : DigitCharacter .. ~~ "," ~~ y : DigitCharacter .. ~~ 
       ")" :> ConvertBase1ToBase0[x, y]}];
  Export[fileOut, in3, "Text"]
  ]
 
 
 
 
(* ------------------- Write to a Matlab or C file that removes double quotes ---------------- *) 
RemoveDoubleQuotesFromFile::usage = "RemoveDoubleQuotesFromFile[{dir, file}]: Removes double quotes from the file.";

RemoveDoubleQuotesFromFile[{dir_String, file_String}] :=
Module[{str, str1},
 str = Import[FileNameJoin[{dir, file}], "Text"];
 str1 = StringReplace[str, {"\"" -> ""}];
 Export[FileNameJoin[{dir, file}], str1, "Text"]
 ]
 
ReplaceEqualWithCEqualInFile::usage = "ReplaceEqualWithCEqualInFile[{dir, file}]: Replace == by = in the file."; 
 
ReplaceEqualWithCEqualInFile[{dir_String, file_String}] :=
Module[{str, str1},
 str = Import[FileNameJoin[{dir, file}], "Text"];
 str1 = StringReplace[str, {"==" -> " = "}];
 Export[FileNameJoin[{dir, file}], str1, "Text"]
 ]
